/* note: IF both HIDE_UNALLOWED_NEXT_BUTTON and MUTE_UNALLOWED_NEXT_BUTTON_SOUND below are false,
THEN when the "Next" button is disallowed (via allowNextButton(false)), 
AND someone clicks on it, 
THEN its clickNext() action will be selectively uneffective (validation done, but no goNextButton(), as expected)
BUT it will beep AND this can be misleading for the user
*/

HIDE_UNALLOWED_NEXT_BUTTON = false;  // hint: false is preferred 
/* hides the button via the nextButtonVisible state, which translates to 'visibility' prop of PcButton "Next"
BUT it stays visible for ~500ms before being actually invisible
meanwhile, it is not clickable, so it is visible as living-dead but not making sound.
SO on fast machines, that's ok.
BUT on slow machines, we will see the button vanish or reappear with an unpredictable subsecond delay, which can be misleading for the user
Note: seeing how the code already uses it, I added a call to forceUpdate(), indeed it speeds up hiding the button. to be discussed with Alban.
ANYWAY, as an alternative: 
*/
MUTE_UNALLOWED_NEXT_BUTTON_SOUND = true; // hint: true is preferred
/* the next button visibility is unchanged   
BUT we immediately allow or disallow it to make sound via new method PcButton.muteSound()
this produces the same effect than HIDE_UNALLOWED_NEXT_BUTTON (at least on fast machines)
AND is a more clever option 
*/

class PcScreenInfos extends PcScreen // PCSI only uses PcForms and buttons 
{
  constructor(props)
  {
    super(props);

    this.buttons = [];
    this.buttons["previous"] = React.createRef();
    this.buttons["next"] = React.createRef();

    this.enableNextButton = false;
    this.form = React.createRef();
    this.alertWindow = React.createRef();
    this.docWindow = React.createRef();

    this.focusTimeout = null;
    this.resetTimeout = null;

    this.registerButton(this.buttons["previous"], PcConfig.screen.infos.previousButton.title);
    this.registerButton(this.buttons["next"], PcConfig.screen.infos.nextButton.title);

    this.state = {alertActive:false, alertWindowTitle:"", docActive:false, docWindowTextFile:"", nextButtonVisible:!HIDE_UNALLOWED_NEXT_BUTTON};
  }

  onEndProgress(x) //override from PcScreen. called from PcProgressBar.doCount() when expired. do not call super.onEndProgress() (PcScreen parent method), see what clickNext() does here
  {
    //super.onEndProgress(x);
    PcDbg.log(CRASHDBG, `SCREENINFOS SECIFIC enter as ${this.constructor["name"]}`);
    this.clickNext("onEndProgress", /*bypass_validation=*/true); // when the progress bar is done, don't need to validate form element, bypass it.  
  }
  normalizeButtons()
  {
    setTimeout(()=>{this.normalizeButtonsWidth()}, 10);
  }

  componentDidStuff()
  {
    if (this.props.active)
    {
      super.componentDidStuff();
      this.normalizeButtons();
      ReactDOM.findDOMNode(this).focus();
    }
  }
  /*onClick() // proof that onClick override on PcScreen child class works
  {
    PcDbg.log(CRASHDBG, "Infos onClick");
    super.onClick();
  }
  */
  componentDidMount()
  {
    this.componentDidStuff();
  }
  componentDidUpdate()
  {
    this.componentDidStuff();
  }

  onClickAlertWindow()
  {
    this.setState({alertActive:false});
  }

  onClickDocWindow()
  {
    this.setState({docActive:false});
  }

  clickPrevious()
  {
    if (this.form.current.currentElement==0)
    {
      //PcConfig.sounds.buttonPrevious.play();
      super.onEndProgress(); // stop pre-bar timer and progress bar if any. go back to prev screen
      this.props.goPrevScreen();
    }
    else {
      super.onEndProgress(/*restart =*/true); // stop pre-bar timer and progress bar. restart them. go bback to previous form element at this same screen

      this.form.current.goPrevious();  
      let b = this.form.current.getNextButton();
      this.buttons["next"].current.updateLabel(b.title, null, b.nextArrow);
    }

  }

  allowNextButton(yesno) // PcScreenInfos has the jurisdiction to control the state enabled/disabled of the "next" button
  {
    PcConfig.dbg(CRASHDBG, "enter with yesno", yesno);
    this.enableNextButton = yesno; // if true, disables clickNext:goNextButton stuff but doesn't disable clickNext:validate() stuff, this is intended
    if (HIDE_UNALLOWED_NEXT_BUTTON) {
       this.showNextButton(yesno);
     }  else {
       if (MUTE_UNALLOWED_NEXT_BUTTON_SOUND && this.buttons["next"].current) {
         this.buttons["next"].current.muteSound(!yesno);
       }
     }
     // disables the button to avoid hearing clicks when not allowed. this comes as a complement to enableNextButton because showNextButton can take a few frames before being taken into account 
    PcConfig.dbg(CRASHDBG, "exiting");
  } 
  clickNext(from, bypass_validation=false) // clickNext DOES the validate (error) check. we must call it to keep error popups.
  {
    let error;


      error = this.form.current.validate();

    PcConfig.dbg(CRASHDBG, "entering from", from, "with error =*", error, "*");
    if ((!bypass_validation) && (error != PcFormErrorMessage.OK.str_msg)) { // got an error message? show it
      this.alertWindow.current.updateLabel(error);
      this.setState({alertActive:true});
      this.forceUpdate();
    } else if (this.enableNextButton|| bypass_validation) { // no popup error expected but is the form actually ready ? ie has onAnsweredQuestion been called and related setMemory on questions actually done?
        if (this.form.current.goNext())
        {
        PcConfig.dbg(CRASHDBG, "launching goNextScreen");
        super.onEndProgress(); // stop pre-bar timer and progress bar if any
        this.props.goNextScreen();
        }
        else {
          super.onEndProgress(true); //  stop pre-bar timer and progress bar if any. restart both
          let b = this.form.current.getNextButton();
          PcConfig.dbg(CRASHDBG, "update button label", b.title);
          this.buttons["next"].current.updateLabel(b.title, null, b.nextArrow);
          
        }
    } else { // nope, the form is not ready, this click will go nowhere
        PcConfig.dbg(true, "Next button not allowed! Please wait for the form to be ready!");
    }
    PcConfig.dbg(CRASHDBG, "exiting");
  }

  forceNext()
  {
    console.log("FORCE NEXT");
    if (this.form.current.goNext())
    {
      this.props.goNextScreen();
    }
    else
    {
      let b = this.form.current.getNextButton();
      this.buttons["next"].current.updateLabel(b.title, null, b.nextArrow);
    }
  }

  clickDoc(textFile)
  {
    console.log("clickdoc");
    this.docWindow.current.updateTextFile(textFile);
    this.setState({docActive:true});
    this.forceUpdate();
  }

  reset()
  {
    this.form.current.reset();
  }
  showNextButton(yesno)
  {
    let DEBUG_TIMEOUT = 0; // hint: adjust to 1000 to see the setState({nextButtonVisible:yesno}) latency
    PcConfig.dbg(CRASHDBG, "entering with show", yesno, "whereas state is", this.state.nextButtonVisible);
    if (this.state.nextButtonVisible != yesno) {
      PcConfig.dbg(CRASHDBG, "SWITCHING TO", yesno);

      setTimeout( () => {
        this.setState({nextButtonVisible:yesno});
      }, yesno ? DEBUG_TIMEOUT : 0); // adjust timeout just to prove to yourself.
      /* 
      "nextButtonVisible:false" works: button is disabled. but not instanstaneously invisible. 
      so a timeout of 0 will actually instantly disable the button (not clickable anymore, no parasitic beeps), but it won't disappear.
      it will disappear on slower machines... maybe that's not what we want.
      maybe ultimately the goal is for the button is to be visible but not to beep while it is not ready
      Note: there are calls to forceUpdate() everywhere, they force a rendering.
      normally it shouldn't be used (TM) but it speed things up, see other existing calls in the code
      */
     this.forceUpdate();
    }
    PcConfig.dbg(CRASHDBG, "exiting");
  }
  render()
  {
    let itemVerticalMarginDiffPercent = Math.max(0, PcConfig.screen.infos.listingItemVerticalMarginPercent - 100);
    let totalItemExcessPx = Math.round(PcConfig.global.maxListingEntryCount * PcConfig.global.defaultItemSpacing * itemVerticalMarginDiffPercent / 100);

    if (!this.props.active)
    {
      clearTimeout(this.focusTimeout);
      clearTimeout(this.resetTimeout);
      this.state.docActive = false;
      return null;
    }

    if (this.props.active)
    {
      if (PcConfig.getMemory("formShouldReset"))
      {
        PcConfig.setMemory("formShouldReset", false);
        this.resetTimeout = setTimeout(()=>{this.reset();}, 10);
        for (let i=0; i<PcConfig.screen.infos.questions.length; i++)
        {
          PcConfig.setMemory("question"+i, "");
        }
      }
    }

    this.focusTimeout = setTimeout(()=>{ReactDOM.findDOMNode(this).focus(); this.form.current.onKeyPressed("")}, 20);

    return e('div',
      {
        className:this.state.className,
        index:this.props.index,
        style:
        {
          display: (this.props.active ? 'block' : 'none'),
          fontFamily:PcConfig.scenario.fontFamily,
          animation:this.props.noAnimation?"":this.state.animationStyle
        },
        onClick:()=>{this.onClick();}, /* allows this screen to listen to clicks and do stuff (for pre-bar timer and progress bar for example) */
        /*onMouseMove:()=>{this.onClick();},*/
        onTouchEnd:()=>{this.onClick();},
      },
      this.bar, //fixme
      e(PcLabel,
        {
          label:PcConfig.screen.infos.title,
          title:true,
          width:this.props.width,
          height:this.props.height*(0.1),
          x:0,
          y:this.props.height*0.03,
          fontSize:PcConfig.scenario.fontSizeBig,
          color: PcConfig.scenario.fontColor,
          showDelay:"0,1s"
        }
      ),
      e(PcForm,
        {
          ref:this.form,
          active:this.props.active,
          keepState:this.props.keepState,
          clickPrevious:()=>this.clickPrevious(),
          clickNext:()=>this.clickNext("PcScreenInfos:PcForm"),
          forceNext:()=>this.forceNext(),
          allowNextButton:(yesno)=>this.allowNextButton(yesno),
          onClickFormDoc:(textFile)=>this.clickDoc(textFile),
          goNextScreen:this.props.goNextScreen,
          title:PcConfig.screen.infos.title,
          position:"absolute",
          left:this.props.width*(1-PcMode.screen.infos.formW)/2,
          top:Math.max(0, this.props.height*0.1 - totalItemExcessPx),
          width:this.props.width*PcMode.screen.infos.formW,
          height:this.props.height*0.75,
          fontFamily:PcConfig.scenario.fontFamily,
          fontSize:PcConfig.scenario.fontSize,
          color: PcConfig.scenario.fontColor,
          centered : true,
        }
      ),
      e('div',
        {
          style:
            {
              position:"absolute",
              width:"100%",
              top:"85%",
              height:"15%",
              display:"inline-flex",
              justifyContent: "space-evenly",
              alignItems:"center",
              alignContent:"space-around"
            }
        },
        e(PcButton,
          {
            ref:this.buttons["previous"],
            label:PcConfig.screen.infos.previousButton.title,
            align:"right",
            onClick:()=>this.clickPrevious(),
            fontColor:PcConfig.screen.infos.previousButton.fontColor,
            borderColor:PcConfig.screen.infos.previousButton.borderColor,
            backgroundColor:PcConfig.screen.infos.previousButton.backgroundColor,
            prevArrow:true,
            sound : PcConfig.sounds.buttonPrevious,
            showDelay:"0.3s"
          }
        ),
        e(PcButton,
          {
            ref:this.buttons["next"],
            visibility: this.state.nextButtonVisible ? "visible" : "hidden",
            label:  PcConfig.screen.infos.questions.length==1 ?  PcConfig.screen.infos.nextButton.titleValidate : PcConfig.screen.infos.nextButton.title,
            fontColor:PcConfig.screen.infos.nextButton.fontColor,
            borderColor:PcConfig.screen.infos.nextButton.borderColor,
            backgroundColor:PcConfig.screen.infos.nextButton.backgroundColor,
            nextArrow:PcConfig.screen.infos.questions.length==1 ? false : true,
            animated:true,
            onClick:()=>this.clickNext("PcScreenInfos:PcButton"),
            sound : PcConfig.sounds.buttonNext,
            showDelay:"0.4s"
          }
        )
      ),
      e(PcAlert,
        {
          ref:this.alertWindow,
          active:this.state.alertActive,
          onClickButton:()=>this.onClickAlertWindow(),
          label:this.state.alertWindowTitle,
          buttonLabel:PcConfig.screen.infos.popButtonLabel,
        }
      ),
      e(PcDoc,
        {
          ref:this.docWindow,
          active:this.state.docActive,
          onClickButton:()=>this.onClickDocWindow(),
          textFile:this.state.docWindowTextFile,
          buttonLabel:PcConfig.screen.infos.popButtonLabel,
        }
      )
    );
  }
}
