#!/usr/bin/env python3

#Test Valina

import sys, os
p = os.path.abspath(os.path.dirname(os.path.realpath(sys.argv[0]))) + '/..'
print(f"p is {p}")
sys.path.append(p) # for common
from common import _sql_exec
import argparse
import comvalina
from time import sleep
from com_tools import *
import err_codes_valina
import signal
import xmltodict
import enum
import simpleflock
import time
from call_function_with_timeout import SetTimeout
CRASHDBG = True
SQLDBG=True
LOOPDBG = False
DBG_TO_FILE = False # super useful in cron jobs

DEFAULT_LOOP_TIME = 1 # unused
SLEEP_TIME_NO_EPT_REQUIRED=5 #is slow/fast enough whenever the configuration changes
SLEEP_TIME_INTERNAL_EXCEPTION=5 #does it have any sense? why not use SLEEP_TIME_NO_EPT_JOBS_FOUND?
SLEEP_TIME_NO_EPT_JOBS_FOUND=1 # good duration
SLEEP_TIME_MODE_EPT_FAILING_DBG=1 #when debugging, go fast
SLEEP_TIME_MODE_EPT_FAILING_PROD=30 #in prod, go as customer asks
SLEEP_TIME_MODE_EPT_FAILING = SLEEP_TIME_MODE_EPT_FAILING_PROD
PAYMENT_HARD_TIMEOUT_DEFAULT=360 #if not specified by backend
class AppError(enum.IntEnum):

	ERR_CODE_EPT_STATUS_OK=0,
	ERR_CODE_SERIAL_PORT_NOT_FOUND=1, #maintenance
	ERR_CODE_EPT_NOT_FOUND=2, #maintenance
	ERR_CODE_EPT_PAYMENT_WAITING=3, # unused
	ERR_CODE_EPT_OR_SERIAL_FAILURE=4, #maintenance
	ERR_CODE_EPT_NOT_REQUIRED=5,
	ERR_CODE_EPT_PAYMENT_OK=6,
	ERR_CODE_EPT_PAYMENT_PURSE_FAILURE=7,
	ERR_CODE_EPT_PAYMENT_AUTHORISATION_FAILURE=8, #unused
	ERR_CODE_EPT_PAYMENT_TIMEOUT=9,
	ERR_CODE_EPT_PAYMENT_DEVICE_CANCEL=10,
	ERR_CODE_INTERNAL_EXCEPTION=11, #maintenance
	ERR_CODE_EPT_LIBRARY_ERROR=12, #maintenance
	ERR_CODE_EPT_UNHANDLED_ERROR=13, #maintenance
	ERR_CODE_EPT_PAYMENT_TAKING_FOREVER=14 #maintenance
	ERR_CODE_EPT_UNHANDLED_PENDING_TRANSACTION=15,
	ERR_CODE_EPT_CANNOT_REACH_SERVER=16

MAINTENANCE_TRIGGERING_STATUSES = [
	AppError.ERR_CODE_SERIAL_PORT_NOT_FOUND,
	AppError.ERR_CODE_EPT_NOT_FOUND,
	AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE,
	AppError.ERR_CODE_INTERNAL_EXCEPTION,
	AppError.ERR_CODE_EPT_LIBRARY_ERROR,
	AppError.ERR_CODE_EPT_UNHANDLED_ERROR,
	AppError.ERR_CODE_EPT_PAYMENT_TAKING_FOREVER	
]

class FullErrorCode:
	def __init__(self, app_error, detailed_error=None):
		self.app_error = app_error
		if detailed_error: 
			self.detailed_error = detailed_error
		else:
			self.detailed_error = app_error

	def __str__(self):
		s = f"FullErrorCode: {self.app_error.name} detailed: {self.detailed_error.name}"
		return s
		
TABLE_PAYMENTS="photocabine_auto_jobs_payment"
TABLE_CABIN_STATUS="photocabine_status"

def _translateErrorToAppError(my_dict, clazz, error):
	dbg(CRASHDBG, f"entering")
	detailed_error = None
	try:
		detailed_error = clazz(error)
		ret = my_dict[error]
	except Exception as ex:
		ret = AppError.ERR_CODE_EPT_UNHANDLED_ERROR
		dbg(CRASHDBG, f"exception occured : {ex}")
	finally:
		if detailed_error is not None:
			dtn = detailed_error.name
		else:
			dtn = f"None! we crashed because of missing error in clazz dict: {error}"
			ret = AppError.ERR_CODE_INTERNAL_EXCEPTION
	dbg(CRASHDBG, f"exiting with ret {ret.name} because detailed_error was {dtn}")
	return FullErrorCode(ret, detailed_error)

def valinaIncidentToAppError(valina_incident):
	dbg(CRASHDBG, "entering")
	clazz = err_codes_valina.ValinaIncident
	VI_TO_APP_ERR= {
		clazz.VALINA_INCIDENT_POSITIVE_MESSAGE: AppError.ERR_CODE_EPT_STATUS_OK,
		clazz.VALINA_INCIDENT_NO_PURSE_AND_TIMEOUT: AppError.ERR_CODE_EPT_PAYMENT_TIMEOUT,		
		clazz.VALINA_INCIDENT_DEVICE_CANCELLATION: AppError.ERR_CODE_EPT_PAYMENT_DEVICE_CANCEL,
		clazz.VALINA_INCIDENT_PURSE_STATE_ERROR : AppError.ERR_CODE_EPT_PAYMENT_PURSE_FAILURE,
		clazz.VALINA_INCIDENT_PURSE_EXPIRED :  AppError.ERR_CODE_EPT_PAYMENT_PURSE_FAILURE,
		clazz.VALINA_INCIDENT_INSUFFICIENT_PURSE_BALANCE :  AppError.ERR_CODE_EPT_PAYMENT_PURSE_FAILURE,
		clazz.VALINA_INCIDENT_PURSE_RED_LIST :  AppError.ERR_CODE_EPT_PAYMENT_PURSE_FAILURE,
		clazz.VALINA_INCIDENT_PURSE_LOCKED_CREDIT :  AppError.ERR_CODE_EPT_PAYMENT_PURSE_FAILURE, 
		clazz.VALINA_INCIDENT_PURSE_LOCKED_DEBIT :  AppError.ERR_CODE_EPT_PAYMENT_PURSE_FAILURE,
		clazz.VALINA_INCIDENT_TECHNICAL_PROBLEM : AppError.ERR_CODE_EPT_CANNOT_REACH_SERVER,
		clazz.VALINA_INCIDENT_UNEXPECTED_MESSAGE: AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE
	}
	dbg(CRASHDBG, f"exiting with incident {valina_incident}")
	return _translateErrorToAppError(VI_TO_APP_ERR, clazz, valina_incident)

def readerStatusToAppError(reader_status): 
	dbg(CRASHDBG, "entering")	
	clazz = err_codes_valina.MagicReaderStatus
	RS_TO_APP_ERR = {
			clazz.READER_STATUS_OK : AppError.ERR_CODE_EPT_STATUS_OK
	}
	return _translateErrorToAppError(RS_TO_APP_ERR, clazz, reader_status)

def valinaExceptionCodeToAppError(exception_code): 
	dbg(CRASHDBG, "entering")	
	clazz = err_codes_valina.ValinaExceptionCode
	VE_TO_APP_ERR = {
			clazz.VALINA_OK: AppError.ERR_CODE_EPT_STATUS_OK,
			clazz.VALINA_ERR_COMM_EMIT_NO_ACK: AppError.ERR_CODE_EPT_NOT_FOUND,
			clazz.VALINA_ERR_COMM_EMIT_WRITE: AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE,# saw it happen, it IS a workaround
			clazz.VALINA_ERR_BAD_POSWARE_VERSION: AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE, # saw it happen, might be a workaround
			clazz.VALINA_ERR_TERMINAL_BUSY: AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE, # saw it happen, might be a workaround
			clazz.VALINA_ERR_TIMEOUT: AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE # saw it happen, might be a workaround
		}
	return _translateErrorToAppError(VE_TO_APP_ERR, clazz, exception_code)


MYPID = os.getpid()
def my_die():
    os.kill(MYPID, signal.SIGKILL)

def parse_args():
    dbg(CRASHDBG, "entering")
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--help', '-h', help='show this help message and exit', action='store_true')
    parser.add_argument('--period', '-l', help='define loop periodicity in floating seconds', type=float, default=DEFAULT_LOOP_TIME)
    parser.add_argument('--error_log', '-e', help='define full path and filename to error_log', type=str, required=False, default="payment_error.log")
    parser.add_argument('--log_file', '-f', help='define full path and filename to log_file', type=str, required=False, default="payment.log")
    parser.add_argument('--lock_file', '-k', help='lock on lockfile', default=None)
    parser.add_argument('--lock_timeout', '-t', help='lockfile timeout', default=0)
    parser.add_argument('--verbose', '-v', help='verbose', action='store_true', default=False)
    args = parser.parse_args()

    if args.help:
        parser.print_help(sys.stdout)
        sys.exit(0)

    #dbg(CRASHDBG, "exiting")
    return args

args = parse_args()

if DBG_TO_FILE:
    from functools import partial
    dbg = partial(fdbg, args.log_file + "-" +str(MYPID)) #works well with cygwin too

def update_table_common(table, d, where_clause):
	ws = "WHERE " + where_clause
	sql_update = f"UPDATE {table} SET {create_set_line(d)}  {ws}"
	dbg(SQLDBG, f"update request: {sql_update}")
	_sql_exec(sql_update, commit=True)


def update_photocabine_auto_jobs_payment(full_err_code, where_clause):
	table = TABLE_PAYMENTS
	d={}
	d["error_code"] = full_err_code.app_error
	d["error_code_hr"] = full_err_code.app_error.name + " (" + full_err_code.detailed_error.name + ")"
	update_table_common(table, d, where_clause)
	


def update_photocabine_status_ept_error_code(full_err_code, where_clause=""):
	#update photocabine status with error ept not required
	table = TABLE_CABIN_STATUS
	d={}
	d["ept_error_code"] = full_err_code.app_error
	d["ept_error_code_hr"] = full_err_code.app_error.name + " (" + full_err_code.detailed_error.name + ")"
	update_table_common(table, d, where_clause)
	


class ValinaEPTHandler():

	def __init__(self):
		return self.reinit()
	
	def reinit(self):
		self.valina = None
		self.kiss_status = None
		self.serial_port = None
		self.xml_dict = None

	def init_ept_lib(self):
		
		dbg(CRASHDBG, "entering")
		if self.valina is not None:
			return FullErrorCode(AppError.ERR_CODE_EPT_STATUS_OK)
		#create new class, will load library
		self.valina = comvalina.Valina()

		#set log file path and name
		self.valina.SetLogFile(f"c:\\log\\debug-valina.txt")

		#set log level to debug
		self.valina.SetLogLevel(4)
		dbg(CRASHDBG, "exiting")
		return FullErrorCode(AppError.ERR_CODE_EPT_STATUS_OK)

	def get_current_state(self):
		dbg(CRASHDBG, "entering")
		ret = self.prepare_ept_session()
		if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
			return ret

		try:
			currentState = self.valina.GetAppState(comvalina.ApplicationNumber.APPLICATION_NFC)
			dbg(CRASHDBG, f"current state is", vars(currentState))
			ret = readerStatusToAppError(currentState.readerStatus)
			
		except err_codes_valina.MyValinaException as ex:		
			ret = valinaExceptionCodeToAppError(ex.value)

			if ret.app_error in [ AppError.ERR_CODE_EPT_OR_SERIAL_FAILURE, AppError.ERR_CODE_EPT_LIBRARY_ERROR ]:
				self.reinit()
	
		dbg(CRASHDBG, f"exiting with ret {ret.app_error.name} {ret.detailed_error.name}")  
		return ret 


	def read_xml(self, config):

		f =  open(f"c:\\www\\xml\\{config}.xml")
		xml_data = f.read()
		self.xml_dict = xmltodict.parse(xml_data)
		

	def set_ept_payment_settings(self, inaction_timeout, transaction_id):
		dbg(CRASHDBG, f"entering with inaction_timeout {inaction_timeout} transaction_id {transaction_id}")
		ret = self.prepare_ept_session()
		if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
			return ret

		dbg(CRASHDBG, f"setting inaction_timeout {inaction_timeout}")
		self.valina.SetWaitCardTimeout(inaction_timeout)
		dbg(CRASHDBG, f"setting transaction id {transaction_id}")
		self.valina.SetTransactionId(str(transaction_id))
		dbg(CRASHDBG, "exiting")
		return FullErrorCode(AppError.ERR_CODE_EPT_STATUS_OK)

	def cancel(self):
		dbg(CRASHDBG, "entering")
		status = self.valina.Cancel()
		ret = valinaIncidentToAppError(status)
		if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
			dbg(CRASHDBG, "cancel error :(", status)
		return ret

	def engage_payment(self, fee_cents, needs_authorisation=True):
		dbg(CRASHDBG, "entering")
		ret = self.get_current_state()
		if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
			dbg(CRASHDBG, f"get_curent_state error :( {ret.app_error.name} {ret.detailed_error.name}")
			return ret

		if needs_authorisation: # pre-payment. "empreinte"
			dbg(CRASHDBG, f"engaging authorisation...")
			status = self.valina.Autorisation(fee_cents)
			#status = 0x1811
			ret = valinaIncidentToAppError(status)
			dbg(CRASHDBG, f"authorisation ended with status {status} -> app_error {ret.app_error.name} {ret.detailed_error.name}")
			if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
				dbg(CRASHDBG, "authorisation error :(")
				return ret
			
			dbg(CRASHDBG, f"engaging record...") # actual payment record. all authorisations require closing record, even if it somehow failed
			status = self.valina.Record(fee_cents)
			ret = valinaIncidentToAppError(status)
			dbg(CRASHDBG, f"record ended with status {status} -> app_error {ret.app_error.name} {ret.detailed_error.name}")
			if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
				dbg(CRASHDBG, "record error :(")
				return ret
			
		else: 
			dbg(CRASHDBG, f"engaging simple payment...")
			status = self.valina.Payment(fee_cents)
			ret = valinaIncidentToAppError(status)
			dbg(CRASHDBG, f"simple payment ended with status {status} -> app_error {ret.app_error.name} {ret.detailed_error.name}")
			if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
				dbg(CRASHDBG, "simple payment error :(")
				return ret
		final_ret = FullErrorCode(AppError.ERR_CODE_EPT_PAYMENT_OK)
		dbg(CRASHDBG, f"exiting normally with ret {final_ret.app_error.name} {final_ret.detailed_error.name}")
		return final_ret

	
	def prepare_ept_session(self, port=None):
		dbg(CRASHDBG, "entering")
		if port:
			self.serial_port  = port
		else:
			
			if not self.serial_port:
				self.serial_port = self.find_port()
		if not self.serial_port:
			return FullErrorCode(AppError.ERR_CODE_SERIAL_PORT_NOT_FOUND)

		dbg(CRASHDBG, f"port is {self.serial_port}")
		ret = self.init_ept_lib()
		if ret.app_error != AppError.ERR_CODE_EPT_STATUS_OK:
			return ret
		dbg(CRASHDBG, f"ept_lib ok")
		if self.kiss_status == 0:
			dbg(CRASHDBG, f"kiss already initialized correctly : status {self.kiss_status}")
			ret = FullErrorCode(AppError.ERR_CODE_EPT_STATUS_OK)
		else: 
			serial_rate = 115200
			dbg(CRASHDBG, f"initializing kiss with rate {serial_rate}")
			self.kiss_status = self.valina.InitializeRs232KISS(self.serial_port)
			if self.kiss_status != 0:
				dbg(CRASHDBG, f"Failed to initialize kiss communication on port {self.serial_port} with status {self.kiss_status}")
				ret = FullErrorCode(AppError.ERR_CODE_EPT_LIBRARY_ERROR)
			else:
				dbg(CRASHDBG, f"kiss init ok")
				ret = FullErrorCode(AppError.ERR_CODE_EPT_STATUS_OK)

		dbg(CRASHDBG, f"exiting with status {ret.app_error.name} {ret.detailed_error.name}")
		return ret



	def find_port(self):
		dbg(CRASHDBG,"entering")
		p = None
		l = list_ports()
		if len(l) > 0: 
			p = l[0]
		dbg(CRASHDBG, "port is", p)
		return p


		
	def loop(self):
		config_requires_ept = False
		stay_awake_once = True
		while True : 
			dbg(CRASHDBG, "loop")
			#preliminary: get config
			mycursor2 = _sql_exec(f"SELECT id, config, ept_error_code, ept_error_code_hr from {TABLE_CABIN_STATUS} ORDER BY id DESC LIMIT 1")
			for d in mycursor2:
				my_dict2 = d
			id_photocabine_status = my_dict2["id"]
			xml_config = my_dict2["config"]
			ept_error_code = my_dict2["ept_error_code"]
			ept_error_code_hr = my_dict2["ept_error_code_hr"]
			dbg(CRASHDBG, f"found xml config {xml_config}")
			try: 			
				self.read_xml(xml_config)
				# dbg(CRASHDBG, f"xml_dict is {self.xml_dict}")
				config_requires_ept = (False, True)[self.xml_dict["root"]["general"]["payment"]["active"] == "1"]

				dbg(CRASHDBG, f"requires ept? {config_requires_ept}")
				#0: check if current config in DB requires EPT
				# it doesn't? cool off and loop
				if not config_requires_ept: 
					update_photocabine_status_ept_error_code(FullErrorCode(AppError.ERR_CODE_EPT_NOT_REQUIRED), f"id = {id_photocabine_status}")
					dbg(CRASHDBG, f"no ept required, sleeping {SLEEP_TIME_NO_EPT_REQUIRED} seconds")
					sleep(SLEEP_TIME_NO_EPT_REQUIRED)
					continue

				# 0: precondition: check if the ept has previously failed or or not 
				do_err_sleep = False
				do_get_state = False
				if ept_error_code is None:
					do_get_state = True
				elif ept_error_code in  MAINTENANCE_TRIGGERING_STATUSES:
					do_err_sleep = True
					do_get_state = True
					
				if ept_error_code == AppError.ERR_CODE_EPT_PAYMENT_TAKING_FOREVER.value: #terminal never answered? try to cancel the transaction 
					cancel_ret = self.cancel()
					dbg(CRASHDBG, f"tried cancelling the unfinished transaction. cancel_ret is {cancel_ret}")

				if do_err_sleep:
					dbg(CRASHDBG, f"sleeping {SLEEP_TIME_MODE_EPT_FAILING} seconds because of a previously met maintenance triggering error {ept_error_code}/{ept_error_code_hr}")
					if stay_awake_once:
						dbg(CRASHDBG, f"first loop run, stay awake once instead of sleeping")
					else:
						sleep(SLEEP_TIME_MODE_EPT_FAILING)

				if stay_awake_once:
					dbg(CRASHDBG, f"first loop run, get state once")
					do_get_state = True
					stay_awake_once = False

				if do_get_state:
					ret = self.get_current_state()
					#update photocabine status ept error code
					update_photocabine_status_ept_error_code(ret, f"id = {id_photocabine_status}")
					if ret.app_error in MAINTENANCE_TRIGGERING_STATUSES:
						dbg(CRASHDBG, f"EPT still in error! was {ept_error_code}/{ept_error_code_hr}, now is {ret.app_error.name} {ret.detailed_error.name}")					
						continue

				# 1: from photocabin status' ept_error_code, tpe is supposed to be working:  find pending payment job
				sql_query = f"SELECT * from {TABLE_PAYMENTS} where worker_pid is NULL and error_code is NULL and config_record = '{xml_config}' ORDER BY date asc LIMIT 1" 
				_sql_exec(sql_query, commit=True)

				mycursor = _sql_exec(sql_query)
				if mycursor.rowcount:  
					for d in mycursor:
						my_dict = {}
						payment_id = d["id"]
						# pending payment job, handle it by setting the id_photocabin_status id, config_record, fee_cents_record and worker_pid
						my_dict["worker_pid"] = MYPID
						
						if False: # get payment settings # from xml
							inaction_timeout = int(self.xml_dict["root"]["general"]["payment"]["waitEPTTime"])
							fee_cents = int(float(self.xml_dict["root"]["general"]["payment"]["fee"]) * 100) #cents
						else:  # get from database 'record' fields
							fee_cents = int(d["fee_cents_record"])
							MARGIN = 8
							inaction_timeout = max(0, int(d["ttl_seconds_record"]) - MARGIN) 
							try: 
								payment_hard_timeout = int(self.xml_dict["root"]["general"]["payment"]["waitEPTTimeMax"])
							except: 
								payment_hard_timeout = 360
							payment_hard_timeout -= MARGIN
							payment_hard_timeout = max(payment_hard_timeout, 0)
						update_table_common(TABLE_PAYMENTS, my_dict, f"id = {payment_id}")

						#2. set payment wait card inaction_timeout.
						self.set_ept_payment_settings(inaction_timeout, payment_id)

						#3. engage payment # wait for it for payment_hard_timeout seconds, fixed time
						func_with_timeout = SetTimeout(self.engage_payment, timeout=payment_hard_timeout)
						# call the function
						is_done, is_func_timeout, err_message, temp_ret = func_with_timeout(fee_cents, True)
						dbg(CRASHDBG, f"is_done: {is_done} is_func_timeout {is_func_timeout} temp_ret {temp_ret}")
						if is_func_timeout: #ept payment is taking forever.
							# this will put is in maintenance mode / error check every 30s, and we will cancel
							ret = FullErrorCode(AppError.ERR_CODE_EPT_PAYMENT_TAKING_FOREVER)
						elif is_done:
							ret = temp_ret
						
						#4 report results, the repeat / stop logic is handled by queue_job_payment.php
						update_photocabine_auto_jobs_payment(ret, f"id = {payment_id}")
						
						#finally, decide if current error needs to be reflected in photocabine_status_ept_error_code or not
						if ret.app_error in MAINTENANCE_TRIGGERING_STATUSES:
							dbg(CRASHDBG, f"major error {ret}, we will ask for maintenance")
							final_ret = ret #yes
						else:
							final_ret = FullErrorCode(AppError.ERR_CODE_EPT_STATUS_OK) #no
							dbg(CRASHDBG, f"minor error {ret} will be overriden by {final_ret}")
						update_photocabine_status_ept_error_code(final_ret, f"id = {id_photocabine_status}")
						'''
						if ret.app_error == AppError.ERR_CODE_EPT_PAYMENT_OK:
							dbg(CRASHDBG, f"PAYMENT OK. ADD ONE CREDIT TO {TABLE_CABIN_STATUS}")
							my_dict = {"payment_credit": 1}
							update_table_common(TABLE_CABIN_STATUS, my_dict, f"id= {id_photocabine_status}")
						'''


				else: 	
					# no pending payment jobs
					dbg(CRASHDBG, f"no payment jobs found, sleeping {SLEEP_TIME_NO_EPT_JOBS_FOUND} seconds")
					#and sleep
					sleep(SLEEP_TIME_NO_EPT_JOBS_FOUND)

			except Exception as ex:
				dbg(True, f"Exception: {ex}, sleeping {SLEEP_TIME_INTERNAL_EXCEPTION} seconds")
				update_photocabine_status_ept_error_code(FullErrorCode(AppError.ERR_CODE_INTERNAL_EXCEPTION), f"id = {id_photocabine_status}")
				sleep(SLEEP_TIME_INTERNAL_EXCEPTION)


def garbage_code():
	#create new class, will load library
	valina = comvalina.Valina()

	#init using Rs232 KISS
	status = valina.InitializeRs232KISS(f"COM10")
	#status = valina.InitializeTcpServer(2009) - same, using tcp/ip
	if status != 0:
		print(f"Failed to initialize communication with err", status)
		exit()

	#set log file path and name
	valina.SetLogFile(f"c:\\log\\debug-valina.txt")

	#set log level to debug
	valina.SetLogLevel(4)

	#set payment wait card timeout. default is 20 secs
	valina.SetWaitCardTimeout(20)

	#set product delivery timeout
	valina.SetProductDeliveryTimeout(60)

	#dummy transaction id
	valina.SetTransactionId(f"ABCD12345678")

	#get current state
	currentState = valina.GetCurrentState()
	print(f"current state is", currentState.name)

	#get library version
	currentVersion = valina.GetVersion()
	print(f"Version is", currentVersion)
	if 0:
		#payment - single step
		status = valina.Payment(1)
		print(f"Payment returned", status)
	else:
		#payment - two steps
		status = valina.Autorisation(1)
		if status == 0:
			#	record payment, same amount
			valina.Record(1)
		else:
			print(f"Autorisation failed with err", status)

	#get customer receipt
	try:
		ticketText = valina.GetTicket(comvalina.TypeTicket.CUSTOMER_RECEIPT)
		print(f"Customer receipt=", ticketText)
	except Exception as e:
		print(f"Get ticket failed with err", e)

	if 0:
		# force app call col/par
		status = valina.ForceAppCall(comvalina.ApplicationNumber.APPLICATION_NFC, comvalina.CallType.CLEARING_CALL)
		print(f"ForceAppCall returned", status)

	#set OID value(int)
	status = valina.SetOID(f"1.3.6.1.4.1.18270.3.2.3.1.60", 0, 1)
	print(f"SetOID technology selection to 1 returned", status)

	#get OID value(int)
	try:
		oidValue = valina.GetOIDInt(f"1.3.6.1.4.1.18270.3.2.3.1.60", 0)
		print(f"Get OID suceeded, value=", oidValue)
	except Exception as e:
		print(f"Get OID failed with exception", e)

	#get OID value (string)
	try:
		oidValue = valina.GetOID(f"1.3.6.1.4.1.18270.3.2.1.12.1.1.1.2", 1)
		print(f"GetOID interface name =", oidValue)
	except Exception as e:
		print(f"Get OID failed with exception", e)

	#test sendcommand
	#try:
	#	reply = valina.SendCommand(5001, "226120", 30*1000)
	#	print(f"SendCommand() suceeded, returned", reply)
	#except Exception as e:
	#	print(f"SendCommand() failed with exception", e)

	#get application state
	try:
		appState = valina.GetAppState(comvalina.ApplicationNumber.APPLICATION_NFC)
		print(f"AppState dateTelecol is", appState.dateTelecol)
	except Exception as e:
		print(f"AppState failed with exception", e)


def test():

	if False:
		vvv = ValinaEPTHandler()
		timeout = 30
		fee_cents = 1
		transaction_id = "123"

		vvv.get_current_state()
		vvv.set_ept_payment_settings(timeout, transaction_id)
		
		vvv.engage_payment(fee_cents, True)
	else: 
		if args.lock_file:
			try:
				dbg(args.verbose or CRASHDBG, f"trying to lock on {args.lock_file} with timeout {args.lock_timeout}")
				with simpleflock.SimpleFlock(args.lock_file, args.lock_timeout):
					vvv = ValinaEPTHandler()
					vvv.loop()
			except BlockingIOError as ex:
				dbg(True, f"cannot lock {args.lock_file}: already in use, good news")
				sys.exit(1)
		else:
			vvv = ValinaEPTHandler()
			vvv.loop()

test()